@@ -3,7 +3,7 @@ use crate::query::{DenseIter, QueryBase, QueryFilter, QueryModifier, SparseIter}
 use crate::storage::Entity;
 use crate::utils::EntityIterator;
 
-/// Iterator over grouped or ungrouped component storages.
+/// Iterator over components of grouped or ungrouped `ComponentStorage`s.
 pub enum Iter<'a, B, I, E, F>
 where
     B: QueryBase<'a>,
@@ -11,9 +11,9 @@ where
     E: QueryModifier<'a>,
     F: QueryFilter,
 {
-    /// Iterator over ungrouped component storages.
+    /// Iterator over components of ungrouped `ComponentStorage`s.
     Sparse(SparseIter<'a, B, I, E, F>),
-    /// Iterator over grouped component storages. Extremely fast.
+    /// Iterator over components of grouped `ComponentStorage`s. Extremely fast.
     Dense(DenseIter<'a, B, F>),
 }
 
@@ -25,7 +25,7 @@ where
     F: QueryFilter,
 {
     /// Creates a new iterator from the given query parts.
-    pub fn new(base: B, include: I, exclude: E, filter: F) -> Self {
+    pub(crate) fn new(base: B, include: I, exclude: E, filter: F) -> Self {
         if query::is_trivial_group::<B, I, E>() {
             let (iter_data, base) = base.split_dense();
 
@@ -33,25 +33,23 @@ where
         } else {
             match query::group_range(&base, &include, &exclude) {
                 Ok(range) => {
-                    let (base_data, base) = base.split_dense();
-                    let iter_data = base_data.with_range(range);
+                    let (mut iter_data, base) = base.split_dense();
 
-                    unsafe { Self::Dense(DenseIter::new_unchecked(iter_data, base, filter)) }
+                    unsafe {
+                        iter_data.entities = iter_data.entities.get_unchecked(range);
+                        Self::Dense(DenseIter::new_unchecked(iter_data, base, filter))
+                    }
                 }
                 Err(_) => {
-                    let (base_data, base) = base.split_sparse();
-                    let (include_data, include) = include.split_modifier();
+                    let (mut iter_data, base) = base.split_sparse();
+                    let (include_entities, include) = include.split_modifier();
                     let (_, exclude) = exclude.split_modifier();
 
-                    let iter_data = if let Some(include_data) = include_data {
-                        if base_data.entities.len() <= include_data.entities.len() {
-                            base_data
-                        } else {
-                            include_data
+                    if let Some(entities) = include_entities {
+                        if entities.len() < iter_data.entities.len() {
+                            iter_data.entities = entities;
                         }
-                    } else {
-                        base_data
-                    };
+                    }
 
                     Self::Sparse(SparseIter::new(iter_data, base, include, exclude, filter))
                 }
