@@ -156,7 +156,7 @@ impl ComponentStorage {
     /// Returns the address of the component mapped to `entity`.
     pub fn get_mut(&mut self, entity: Entity) -> *mut u8 {
         match self.sparse.get_index(entity) {
-            Some(index) => unsafe { self.components.get_unchecked(index) },
+            Some(index) => unsafe { self.components.get_unchecked_mut(index) },
             None => ptr::null_mut(),
         }
     }
@@ -185,7 +185,7 @@ impl ComponentStorage {
 
         unsafe {
             Some((
-                self.components.get_unchecked(index),
+                self.components.get_unchecked_mut(index),
                 self.ticks.get_unchecked_mut(index),
             ))
         }
@@ -196,11 +196,6 @@ impl ComponentStorage {
         self.sparse.contains(entity)
     }
 
-    /// Returns the index in the dense vector of `entity`.
-    pub fn get_index(&self, entity: Entity) -> Option<usize> {
-        self.sparse.get_index(entity)
-    }
-
     /// Removes all entities and components in the storage.
     pub fn clear(&mut self) {
         self.sparse.clear();
@@ -209,20 +204,6 @@ impl ComponentStorage {
         self.components.clear();
     }
 
-    /// Swaps the entities at the given dense indexes.
-    pub fn swap(&mut self, a: usize, b: usize) {
-        let sparse_a = self.entities[a].index();
-        let sparse_b = self.entities[b].index();
-
-        unsafe {
-            self.sparse.swap_unchecked(sparse_a, sparse_b);
-            self.components.swap_unchecked(a, b);
-        }
-
-        self.entities.swap(a, b);
-        self.ticks.swap(a, b);
-    }
-
     /// Returns the number of components in the storage.
     pub fn len(&self) -> usize {
         self.components.len()
@@ -271,8 +252,28 @@ impl ComponentStorage {
         (
             self.sparse.as_view(),
             self.entities.as_slice(),
-            self.components.as_ptr(),
+            self.components.as_mut_ptr(),
             self.ticks.as_mut_slice(),
         )
     }
+
+    /// Returns the index in the dense vector of `entity`.
+    pub(crate) fn get_index(&self, entity: Entity) -> Option<usize> {
+        self.sparse.get_index(entity)
+    }
+
+    /// Swaps the entities at the given indexes without checking if the indexes
+    /// are valid.
+    pub(crate) unsafe fn swap_unchecked(&mut self, a: usize, b: usize) {
+        debug_assert!(a < self.entities.len());
+        debug_assert!(b < self.entities.len());
+
+        let sparse_a = self.entities.get_unchecked(a).index();
+        let sparse_b = self.entities.get_unchecked(b).index();
+        self.sparse.swap_unchecked(sparse_a, sparse_b);
+
+        self.components.swap_unchecked(a, b);
+        self.entities.swap(a, b);
+        self.ticks.swap(a, b);
+    }
 }
