@@ -1,6 +1,6 @@
-use crate::components::{Component, ComponentSet, ComponentStorages};
+use crate::components::{Component, ComponentSet, ComponentStorages, ComponentStoragesIter};
 use crate::layout::Layout;
-use crate::resources::{Resource, ResourceStorage};
+use crate::resources::{Resource, ResourceStorage, ResourceStorageIter};
 use crate::storage::{ComponentStorage, Entity, EntityStorage};
 use crate::utils::{ChangeTicks, NonZeroTicks, Ticks};
 use crate::world::{BorrowWorld, NoSuchEntity, TickOverflow};
@@ -22,7 +22,7 @@ impl WorldId {
     }
 }
 
-/// Container for component storages and entities.
+/// Container for entities, components and resources.
 pub struct World {
     pub(crate) id: WorldId,
     pub(crate) tick: NonZeroTicks,
@@ -84,6 +84,12 @@ impl World {
         self.storages.is_registered(component_type_id)
     }
 
+    /// Returns an iterator over all registered storages and the`TypeId`s of the
+    /// components they hold.
+    pub fn storages(&mut self) -> ComponentStoragesIter {
+        self.storages.iter()
+    }
+
     /// Creates an `Entity` with the given `components` and returns it.
     pub fn create_entity<C>(&mut self, components: C) -> Entity
     where
@@ -283,6 +289,12 @@ impl World {
         self.resources.clear();
     }
 
+    /// Returns an iterator over all resources stored in the `World` and their
+    /// `TypeId`s.
+    pub fn resources(&mut self) -> ResourceStorageIter {
+        self.resources.iter()
+    }
+
     /// Removes all entities, components and resources from the `World`.
     pub fn clear(&mut self) {
         self.entities.clear();
